CAFu = Rock:NewAddon("CAFu", "LibFuBarPlugin-3.0", "LibRockConfig-1.0", "LibRockDB-1.0");
local L = Rock("LibRockLocale-1.0"):GetTranslationNamespace("FuBar_CAFu")

-- DEBUG ONLY
L:SetStrictness(true);

CAFu:SetDatabase("FuBar_CAFuDB");
CAFu:SetDefaultProfile("char");
CAFu:SetDatabaseDefaults('char', {
	trackEventTypes = {
		raid	= true,
		dungeon	= true,
		pvp		= true,
		meeting	= true,
		other	= true
	},
	eventSort	= 'time'
});

function CAFu:OnInitialize()
	self:SetFuBarOption('iconPath', [[Interface\AddOns\FuBar_CAFu\icon]]);
	self:SetFuBarOption('hasIcon', true);
	
	self:SetFuBarIcon([[Interface\AddOns\FuBar_CAFu\icon]]);
    self:SetFuBarText('CAFu');
end;

-- using LibRockConfig-1.0, this will automatically
-- open with the FuBar plugin when right-clicked.
CAFu:SetConfigTable({
	name = "FuBar_CAFu",
	desc = L["CAFu desc"],
	type = "group",
	args = {
		g1 = {
    		name = L["Event types"],
    		desc = L["Event types desc"],
    		type = 'group',
    		args = {
    			raid = {
    				name = L["Raids"],
    				desc = L["Track raids"],
    				order = 1,
    				type = 'boolean',
					get = function() return CAFu.db.char.trackEventTypes.raid end,
					set = function(v) CAFu.db.char.trackEventTypes.raid = v end,
				},
    			dungeon = {
    				name = L["Dungeons"],
    				desc = L["Track dungeons"],
    				order = 2,
    				type = 'boolean',
					get = function() return CAFu.db.char.trackEventTypes.dungeon end,
					set = function(v) CAFu.db.char.trackEventTypes.dungeon = v end,
				},
    			pvp= {
    				name = L["PvP"],
    				desc = L["Track pvp"],
    				order = 3,
    				type = 'boolean',
					get = function() return CAFu.db.char.trackEventTypes.pvp end,
					set = function(v) CAFu.db.char.trackEventTypes.pvp = v end,
				},
    			meeting = {
    				name = L["Meetings"],
    				desc = L["Track meetings"],
    				order = 4,
    				type = 'boolean',
					get = function() return CAFu.db.char.trackEventTypes.meeting end,
					set = function(v) CAFu.db.char.trackEventTypes.meeting = v end,
				},
    			other = {
    				name = L["Other"],
    				desc = L["Track other"],
    				order = 5,
    				type = 'boolean',
					get = function() return CAFu.db.char.trackEventTypes.other end,
					set = function(v) CAFu.db.char.trackEventTypes.other = v end,
				}
    		},
    		order = 1
    	},
		g2 = {
		    name = L["Other options"],
		    desc = L["Other options desc"],
		    type = 'group',
		    args = {
		    	sort = {
		    		name = L["Sort by"],
		    		desc = L["Sort events by"],
		    		type = "choice",
		    		choices = {
		    			time = L['Date/Time'],
		    			alpha = L['Alphabetically']
		    		},
		    		get = function() return CAFu.db.char.eventSort; end,
		    		set = function(v) CAFu.db.char.eventSort = v; end
		    	},
		    	about = {
		    		name = L["About"],
		    		desc = L["About desc"],
		    		type = "execute",
		    		buttonText = L["About"],
		    		func = function() message('FuBar_CAFu by Illiai of Sylvanas'); end,
		    		confirm = nil
		    	}
		    },
		    order = 2
		}
	}
});


--[[  only called when:
	1. fubar loads
	2. showText or showIcon are toggled ON
]]--
function CAFu:OnUpdateFuBarText()
	self:SetFuBarIcon([[Interface\AddOns\FuBar_CAFu\icon]]);
    self:SetFuBarText('CAFu');
end;

function CAFu.OnUpdateFuBarTooltip()
    GameTooltip:AddLine("CAFu");
    
    -- make sure that the calendar is actually loaded, or else this won't work
	if (not IsAddOnLoaded("Blizzard_Calendar")) then
		UIParentLoadAddOn("Blizzard_Calendar");
	end;
    
    -- Add list of events for today
    GameTooltip:AddLine("Events for today:", 1, 1, 1);
    _, _, day, _ = CalendarGetDate();
    if (CalendarGetNumDayEvents(0, day) == 0) then
    	GameTooltip:AddLine('   '..L["none"], 1, 0, 0);
    else
    	local cHour			= tonumber(date('%H'));
    	local cMin			= tonumber(date('%M'));
    	
    	local CALENDAR_EVENT_TYPES = {
    		[0] = 'other',	-- Apparently this is used by events set by blizzard.
    		[1] = 'raid',
    		[2] = 'dungeon',
    		[3] = 'pvp',
    		[4] = 'meeting',
    		[5] = 'other'
    	}
    	
    	local function sortAlpha(a, b)
			return (a.title < b.title);
		end;
		local function sortTime(a, b)
			if(a.hour == b.hour) then
				if(a.minute == b.minute) then
					return (a.title < b.title);
				else
					return (a.minute < b.minute);
				end;
			else
				return (a.hour < b.hour);
			end;
		end;
    	
    	eventList = {};
    	
    	for i = 1, CalendarGetNumDayEvents(0, day) do
    		local title, hour, minute, calendarType, sequenceType, eventType, texture, modStatus, inviteStatus, invitedBy = CalendarGetDayEvent(0, day, i);
			
    		if(getglobal('CAFu.db.char.trackEventTypes.'..CALENDAR_EVENT_TYPES[eventType]) == false) then
    			-- skip this eventtype
    		else
				if(sequenceType == 'ONGOING') then
					-- ignore
				else
					table.insert(eventList, {title=title, hour=hour, minute=minute});
				end;
			end;
    	end;
    	
    	if(#eventList == 0) then
    		GameTooltip:AddLine('   '..L["none"], 1, 0, 0);
    	else
    		if(CAFu.db.char.eventSort == 'time') then
    			table.sort(eventList, sortTime);
    		else
    			table.sort(eventList, sortAlpha);
    		end;
    		
    		for _, event in ipairs(eventList) do
				event.title = '   '..event.title;
				if (cHour < (event.hour-1) or (cHour < event.hour and cMin < event.minute)) then
					-- we're way early ( > 1 hr)
					GameTooltip:AddDoubleLine(event.title, string.format('%02u:%02u', event.hour, event.minute), 0, 1, 0, 1, 1, 1);
				elseif (cHour > event.hour or (cHour == event.hour and cMin > event.minute)) then
					-- too late
					GameTooltip:AddDoubleLine(event.title, string.format('%02u:%02u', event.hour, event.minute), 1, 0, 0, 1, 1, 1);
				else
					-- just in time, but closing in fast ( < 1 hour to go)
					GameTooltip:AddDoubleLine(event.title, string.format('%02u:%02u', event.hour, event.minute), 1, 1, 0, 1, 1, 1);
				end;
    		end;
    	end;
    end;
    
    GameTooltip:AddLine(' ', 0, 0, 0); -- "blank" line
    
    GameTooltip:AddLine(L['tooltip hint'], 0, 1, 0);
end;

function CAFu:OnFuBarClick(button)
	if (not IsAddOnLoaded("Blizzard_Calendar")) then
		UIParentLoadAddOn("Blizzard_Calendar");
	end;
	Calendar_Toggle();
end;